-- Bug: When you don't join Wintergrasp, the icon remains as "In Progress" even after the battle is over
--              Likely issue: Update not being called (probably state change not called when outside of the battle?)

local WGL = CreateFrame("frame")
_G.WGLite = WGL
WGL:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)

local WGST -- Wintergrasp Start Time
local floorTime = function() return math.floor(GetTime()) end
local WGControl

local WGLDragFrame = CreateFrame("Frame","WGLDragFrame",WintergraspTimer)
WGLDragFrame:SetBackdrop({
    bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
    edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", 
    tile=1, tileSize=16, edgeSize=16, 
    insets={left=5, right=5, top=5, bottom=5}
    })
WGLDragFrame:SetWidth(135)
WGLDragFrame:SetHeight(40)
WGLDragFrame:SetPoint("RIGHT",WintergraspTimer,"RIGHT",5,0)
WGLDragFrame:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
WGLDragFrame:Hide()

function WGL:ADDON_LOADED()
    WGL_Save = WGL_Save or { }
    WGL_Save[GetRealmName()] = WGL_Save[GetRealmName()] or { }
    WGL_TimerLoc = WGL_TimerLoc or { }

    WintergraspTimer:SetMovable(true)
    WintergraspTimer:RegisterForDrag("LeftButton")
    WintergraspTimer.texture:SetTexCoord(0.0625,0.9,0.0625,0.9)
    WintergraspTimer.texture:SetTexture("Interface\\Icons\\Inv_misc_questionmark")
    WintergraspTimer:SetScript("OnUpdate", WGL.WintergraspTimer_OnUpdate)
    WintergraspTimer:SetScript("OnHide", function(self) self:Show() end)
 	WintergraspTimer:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 5, 40) GameTooltip:SetText("Click to drag") end)
    WintergraspTimer:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
    WintergraspTimer:SetScript("OnDragStart", function(self) WGLDragFrame:Show() self:StartMoving() end) -- self:SetFrameStrata("TOOLTIP") WGLDragFrame:SetFrameStrata("FULLSCREEN") 
    WintergraspTimer:SetScript("OnDragStop", function(self) 
        if not MouseIsOver(PVPBattlegroundFrame) or not PVPBattlegroundFrame:IsVisible() then 
            self:SetParent(UIParent)
            WGL_TimerLoc.parent = "UIParent"
        else
            self:SetParent(PVPBattlegroundFrame)
            WGL_TimerLoc.parent = "PVPBattlegroundFrame"
            self:ClearAllPoints()
            self:SetPoint("RIGHT",PVPBattlegroundFrame,"TOPRIGHT",-40,-55)
            WGLDragFrame:Hide()
        end
        self:StopMovingOrSizing()
        self:SetFrameStrata("FULLSCREEN_DIALOG")
        WGLDragFrame:SetFrameStrata("FULLSCREEN") 
        WGL_TimerLoc.point = WintergraspTimer:GetPoint(1)
        end)
        
    if WGL_TimerLoc.parent then
        WintergraspTimer:SetParent(getglobal(WGL_TimerLoc.parent))
        if WGL_TimerLoc.parent == "UIParent" then
            WGLDragFrame:Show()
        end
        WintergraspTimer:SetPoint(WGL_TimerLoc.point)
        WintergraspTimer:SetFrameStrata("FULLSCREEN_DIALOG")
        WGLDragFrame:SetFrameStrata("FULLSCREEN") 
    end
        
    PVPBattlegroundFrame:HookScript("OnHide", function() if not MouseIsOver(PVPBattlegroundFrame) and MouseIsOver(WintergraspTimer) then WintergraspTimer:SetParent(UIParent) end end)

    if WGL_Save[GetRealmName()].startTime and WGL_Save[GetRealmName()].startTime - floorTime() > 0 then
        WGST = WGL_Save[GetRealmName()].startTime
        WGControl = WGL_Save[GetRealmName()].control
        if WGControl then
            WintergraspTimer.texture:SetTexCoord(0.0625,0.9,0.0625,0.9)
            WintergraspTimer.texture:SetTexture(WGControl)
        end
    else
        WGL_Save[GetRealmName()].startTime = nil
        WGL_Save[GetRealmName()].control = nil
    end
    
    self:UnregisterEvent("ADDON_LOADED")
end
WGL:RegisterEvent("ADDON_LOADED")

local update = 10 -- So that we get an update immediately on startup
function WGL:WintergraspTimer_OnUpdate(elapsed)
    update = update + elapsed
    if update > 10 then
        update = 0
        WGL:UpdateStatus()
    end

	local nextBattleTime
    if IsInInstance() then
        if WGST then
            local time = WGST - floorTime()
            if time > 0 then
                nextBattleTime = time
            elseif time > -1200 then
                nextBattleTime = 0;
            else
                WGControl = nil
            end
        end
    else
        nextBattleTime = GetWintergraspWaitTime() or 0
    end
    
    if nextBattleTime then
        if nextBattleTime > 60 then
            WintergraspTimer.text:SetFormattedText(PVPBATTLEGROUND_WINTERGRASPTIMER, SecondsToTime(nextBattleTime, true))
        elseif nextBattleTime > 0 then
            WintergraspTimer.text:SetFormattedText(PVPBATTLEGROUND_WINTERGRASPTIMER, SecondsToTime(nextBattleTime, false))
        else
            WintergraspTimer.text:SetFormattedText(PVPBATTLEGROUND_WINTERGRASPTIMER, WINTERGRASP_IN_PROGRESS)
        end
    else
        WintergraspTimer.text:SetFormattedText(PVPBATTLEGROUND_WINTERGRASPTIMER, "Unknown")
    end
end

function WGL:UpdateStatus()
    if not IsInInstance() then
        if GetWintergraspWaitTime() then
            WGST = floorTime() + GetWintergraspWaitTime()
        else
            WGST = 0
        end
        WGL_Save[GetRealmName()].startTime = WGST
        
        if GetCurrentMapContinent() == 4 then
            WGControl = self:GetCurrentMapControl()
            SetMapToCurrentZone()
            WGL_Save[GetRealmName()].control = WGControl
            WintergraspTimer.texture:SetTexture(WGControl)
        end
    elseif not WGControl then
        WintergraspTimer.texture:SetTexture("Interface\\Icons\\Inv_misc_questionmark")
    end
end

function WGL:GetCurrentMapControl()
    SetMapByID(485)
    for i=1, GetNumMapLandmarks() do
        local _,_,index = GetMapLandmarkInfo(i)
        if index == 46 then
            return "Interface\\Icons\\INV_BannerPVP_02"
        elseif index == 48 then
            return "Interface\\Icons\\INV_BannerPVP_01"
        end
    end
    return "Interface\\Icons\\Achievement_Arena_2v2_7"
end